import os, json
from flask import render_template, send_file, request, redirect, url_for
from flask_login import current_user, login_required
from caching import getJson as get_json
from app import app, login_manager
from .roles import ROLES
from .abstract_blog import Blog
from .error import error_handler
from .models import users_repository
from .html_generator import make_top_bar
from .modules.shop import PS3_STATS, XB360_STATS, astore


errorblog = Blog(
    "errorblog",
    "/admin",
    ROLES.ADMIN.value,
    ROLES.OWNER.value,
    True,
    "Got a new error... Progress!",
    view_permission_integer=ROLES.ADMIN.value,
)
errorblog.register(app)

adminblog = Blog(
    "adminblog",
    "/admin",
    ROLES.ADMIN.value,
    ROLES.OWNER.value,
    True,
    "Something something something dark side...",
    view_permission_integer=ROLES.ADMIN.value,
)
adminblog.register(app)

blogs_mount_point = "/b"


@app.route("/favicon.ico")
def favicon():
    return send_from_directory(
        os.path.join(os.path.dirname(__file__), "static"),
        "favicon.ico",
        mimetype="image/vnd.microsoft.icon",
    )


@app.route("/")
@app.route("/index")
def home():
    @error_handler
    def do_home():
        announcements = get_json(
            "announcements.json",
            "https://dl.dropbox.com/s/vnk3s1yb3veyhom/announcements.json?raw=true",
        )

        with open(announcements) as f:
            announcements = list(reversed(json.load(f)["Announcements"]))
        return render_template(
            "/index.html",
            pagetitle="Arisen Studio",
            title="Arisen Studio",
            subtitle="Browse, Download and Install Mods for PlayStation 3 & Xbox 360",
            underhead_text='Join us on <a href="/discord/">Discord</a></div>',
            navbar=make_top_bar(current_user),
            announcements=announcements,
            ps3_stats=PS3_STATS,
            xb360_stats=XB360_STATS,
            last_updated=astore.last_updated,
        )

    return do_home()


@app.route("/about/")
def about():
    @error_handler
    def do_about():
        return render_template(
            "user/about.html",
            pagetitle="About Arisen Studio",
            title="About Arisen Studio",
            subtitle="The Boring Legal Stuff",
            navbar=make_top_bar(current_user),
            current_user=current_user,
        )

    return do_about()


# @app.route('/blog/')
# def blog():
#   @error_handler
#   def do_blog():
#       posts = blog_manager.get_posts_json(20)
#       return render_template("user/blog.html",
#           pagetitle = "BrewTools Blog",
#           title = "BrewTools Blog",
#           subtitle = "Wow Lyfe is annoying.",
#           users_repository = users_repository,
#           posts = posts,
#           navbar = make_top_bar(current_user)
#       )
#   return do_blog()


@app.route("/discord/")
def discord():
    @error_handler
    def do_discord():
        return render_template(
            "user/discord.html",
            pagetitle="Arisen Studio Discord",
            title="Arisen Studio Discord",
            subtitle="We're here.",
            navbar=make_top_bar(current_user),
        )

    return do_discord()


@app.route("/download/")
def download():
    @error_handler
    def do_download():
        return render_template(
            "user/download.html",
            pagetitle="Download Arisen Studio",
            title="Download Arisen Studio",
            subtitle="Mods made simple.",
            navbar=make_top_bar(current_user),
        )

    return do_download()


# @app.route('/form/')
# def form():
#   @error_handler
#   def do_form():
#       return render_template("user/form.html",
#           pagetitle = "Covid Grapher Web Form",
#           title = "Covid Grapher Web Form",
#           subtitle = "See the rona.",
#           navbar = make_top_bar(current_user)
#       )
#   return do_form()

# @app.route('/graph/about/')
# def about_graph():
#   @error_handler
#   def do_about():
#       return render_template("user/about_api.html",
#           pagetitle = "About Covid Grapher",
#           title = "About Covid Grapher Web API",
#           subtitle = "Quick and easy graphs.",
#           navbar = make_top_bar(current_user)
#       )
#   return do_about()

# @app.route('/latest/')
# def latest_post():
#   @error_handler
#   def do_latest_post():
#       post = blog_manager.get_latest_post()
#       u = users_repository.get_user(post.author_id)
#       if u: username = u.username
#       else: username = "Unknown User"
#       return render_template("user/raw_post.html",
#           title = post.title,
#           desc = post.desc,
#           body = post.body,
#           username = username,
#           post_time = post.creation_friendly
#           )
#   return do_latest_post()

# @app.route('/profile/')
# def get_profile():
#   @error_handler
#   def do_get_profile():
#       if current_user.is_authenticated:
#           return redirect(url_for(f'.get_profile_by_id', **{"id" : current_user.id}))
#       else:
#           return redirect('/index')
#   return do_get_profile()

# @app.route('/profile/user/<id>/')
# def get_profile_by_id(id):
#   @error_handler
#   def do_get_profile_by_id():
#       u = users_repository.get_user(id)

#       blogposts = {}

#       for b in boards:
#           p = [b.get_post_by_id(i) for i in b.get_user_posts(id)]
#           if p:
#               blogposts[b] = p

#       return render_template("user/profile.html",
#           pagetitle = f"{u.username}'s Profile",
#           title = f"{u.username}'s Profile",
#           subtitle = u.flair or "This user doesn't have a flair. How sad :(",
#           user = u,
#           navbar = make_top_bar(current_user),
#           role = ROLES(int(u.role)).name,
#           bio = u.bio or "This user does't have a bio. How sad :(",
#           posts = blogposts,
#           current_user = current_user
#           )

#   return do_get_profile_by_id()

# @app.route('/profile/edit/', methods=['POST'])
# @login_required
# def edit_attr():
#   @error_handler
#   def do_edit_attr():
#       if request.form.get('flair'): return redirect("/profile/flair")
#       if request.form.get('bio'): return redirect("/profile/bio")
#   return do_edit_attr()

# @app.route('/profile/bio/', methods=['GET' , 'POST'])
# @login_required
# def edit_bio():
#   @error_handler
#   def do_edit_bio():
#       if request.method == 'GET':
#           return render_template("user/edit.html",
#               pagetitle = f"Edit Profile Bio",
#               title = f"Edit Profile Bio",
#               subtitle = "Tell the world about yourself. You can use raw HTML here, including embedding iframes. Please keep everything PG13 and keep it reasonable.",
#               navbar = make_top_bar(current_user),
#               edit_type = 'bio'
#           )
#       else:
#           preview = request.form.get('preview')
#           submit = request.form.get('submit')
#           body = request.form.get('body')
#           if submit:
#               users_repository.set_bio(current_user.id, body)

#               return ("Bio set successfully")
#           elif preview:
#               return render_template("user/edit_preview.html",
#                   username = current_user.username,
#                   bio = body,
#                   navbar = make_top_bar(current_user)
#               )
#           else:
#               error = getHTMLErrorString("Flair Edit Error.", "There was an error handling your flair edit request.")
#               return render_template("user/error.html",
#                   title = "Profile Edit Error",
#                   subtitle = "Lost in the mail.",
#                   error=error,
#                   navbar = make_top_bar(current_user)
#               )
#   return do_edit_bio()

# @app.route('/profile/flair/', methods=['GET' , 'POST'])
# @login_required
# def edit_flair():
#   @error_handler
#   def do_edit_flair():
#       if request.method == 'GET':
#           return render_template("user/edit.html",
#               pagetitle = f"Edit Profile Flair",
#               title = f"Edit Profile Flair",
#               subtitle = "Tell the world about yourself. You can use raw HTML here, including embedding iframes. Please keep everything PG13 and keep it reasonable.",
#               navbar = make_top_bar(current_user),
#               edit_type = 'flair'
#           )
#       else:
#           preview = request.form.get('preview')
#           submit = request.form.get('submit')
#           body = request.form.get('body')
#           if submit:
#               users_repository.set_flair(current_user.id, body)

#               return ("Flair set successfully")
#           elif preview:
#               return render_template("user/edit_preview.html",
#                   username = current_user.username,
#                   flair = body,
#                   navbar = make_top_bar(current_user)
#               )
#           else:
#               error = getHTMLErrorString("Flair Edit Error.", "There was an error handling your flair edit request.")
#               return render_template("user/error.html",
#                   title = "Profile Edit Error",
#                   subtitle = "Lost in the mail.",
#                   error=error,
#                   navbar = make_top_bar(current_user)
#               )
#   return do_edit_flair()


# callback to reload the user object
@login_manager.user_loader
def load_user(userid):
    return users_repository.get_user(userid)
